from pbxproj import XcodeProject
import os


def list_swift_files(project_path, target_name):
    """
    Lists Swift files for a target name.

    :param project_path: path to .xcodeproj file
    :param target_name: name of the target
    :return:
    """
    (project_directory, _) = os.path.split(project_path)
    project = XcodeProject.load(os.path.join(project_path, 'project.pbxproj'))
    target = project.get_target_by_name(target_name)

    build_phases = (project.objects[p] for p in target.buildPhases if project.objects[p].isa == 'PBXSourcesBuildPhase')
    build_files = (f for p in build_phases for f in p.files)
    build_files = (project.objects[project.objects[f].fileRef] for f in build_files)
    files = (__full_path(project, file) for file in build_files)
    return [os.path.join(project_directory, file_path) for file_path in files]


def __full_path(project, file_ref):
    folder = ''

    if file_ref.sourceTree == '<group>':
        parent = __parent_group(project, file_ref)

        if parent:
            folder = __full_path(project, parent)
    elif file_ref.sourceTree == '<absolute>':
        folder = '/'

    if 'path' in file_ref:
        return os.path.join(folder, file_ref['path'])
    else:
        return folder


def __parent_group(project, file_ref):
    for group in project.objects.get_objects_in_section('PBXGroup'):
        if group.has_child(file_ref.get_id()):
            return group



import os

from unittest import TestCase

from commands.utils.xcproj import list_swift_files


class XCProjTestCase(TestCase):
    def test_that_files_for_app_target_are_resolved(self):
        """
        Test that files for application target called TestProject are correctly resolved.
        """
        files = list_swift_files('tests/resources/TestProject/TestProject.xcodeproj', 'TestProject')

        absolute_file_path = 'tests/resources/TestProject/TestProject/SomeGroup/Deeply/Nested/TestAbsoluteFile.swift'

        self.assertEqual(set(files), {
            'tests/resources/TestProject/TestProject/SomeGroup/TestController.swift',
            os.path.join(os.getcwd(), absolute_file_path),
            'tests/resources/TestProject/TestProject/ViewController.swift',
            'tests/resources/TestProject/TestProject/AppDelegate.swift'
        })

    def test_that_files_for_framework_target_are_resolved(self):
        """
        Test that files for framework target called Framework are correctly resolved.
        """
        files = list_swift_files('tests/resources/TestProject/TestProject.xcodeproj', 'Framework')

        absolute_file_path = 'tests/resources/TestProject/TestProject/SomeGroup/Deeply/Nested/TestAbsoluteFile.swift'

        self.assertEqual(set(files), {
            'tests/resources/TestProject/TestProject/SomeGroup/TestController.swift',
            os.path.join(os.getcwd(), absolute_file_path),
        })



import os
from subprocess import run

import click
import yaml
from click.exceptions import Exit

from commands.utils.xcproj import list_swift_files


@click.command()
@click.argument('xcodeproj_path')
@click.argument('target_name')
@click.argument('swiftlint_cfg_path')
@click.option('--delete-config/--no-delete-config', default=True)
def lint(xcodeproj_path, target_name, swiftlint_cfg_path, delete_config):
    with open(swiftlint_cfg_path, 'r') as sources:
        sources_config = yaml.safe_load(sources)

        with open('.swiftlint.yml', 'w') as output:
            sources_config['included'] = list_swift_files(xcodeproj_path, target_name)
            yaml.dump(sources_config, output, default_flow_style=False, allow_unicode=True)

        process = run(["swiftlint"])

        if delete_config:
            try:
                os.remove('.swiftlint.yml')
            except OSError:
                click.echo('Cleanup failed, could not remove .swiftlint.yml file', err=True)

        if process.returncode != 0:
            raise Exit(process.returncode)



import os
from shutil import copyfile

from unittest import TestCase
from unittest.mock import patch

import yaml
from click.testing import CliRunner

from commands.lint import lint


class LintTestCase(TestCase):
    def setUp(self):
        self.runner = CliRunner()

    @patch('commands.lint.list_swift_files')
    @patch('commands.lint.run')
    def test_that_linting_invokes_listing_files_with_correct_arguments(self, _, list_files):
        """
        Test that linting invokes listing files with correct arguments.
        """
        list_files.return_value = []
        swiftlint_path = os.path.join(os.getcwd(), 'tests/resources/.swiftlint.yml')

        with self.runner.isolated_filesystem():
            copyfile(swiftlint_path, '.sources.yml')
            _ = self.runner.invoke(lint, ['path_to/Project.xcodeproj', 'SomeTarget', '.sources.yml'])

            list_files.assert_called_once_with('path_to/Project.xcodeproj', 'SomeTarget')

    @patch('commands.lint.list_swift_files')
    @patch('commands.lint.run')
    def test_that_linting_deletes_swiftlint_file(self, run, list_files):
        """
        Test that linting deletes SwiftLint file.
        """
        list_files.return_value = []
        run.return_value.returncode = 0
        swiftlint_path = os.path.join(os.getcwd(), 'tests/resources/.swiftlint.yml')

        with self.runner.isolated_filesystem():
            copyfile(swiftlint_path, '.sources.yml')
            runner = self.runner.invoke(lint, ['Project.xcodeproj', 'SomeTarget', '.sources.yml'])

            self.assertEqual(0, runner.exit_code)
            self.assertFalse(os.path.exists('.swiftlint.yml'))

    @patch('commands.lint.list_swift_files')
    @patch('commands.lint.run')
    def test_that_linting_appends_included_files_to_swiftlint_config(self, _, list_files):
        """
        Test that linting appends included files to SwiftLint config.
        """
        list_files.return_value = ['A', 'B', 'C']
        swiftlint_path = os.path.join(os.getcwd(), 'tests/resources/.swiftlint.yml')

        with self.runner.isolated_filesystem():
            copyfile(swiftlint_path, '.sources.yml')
            _ = self.runner.invoke(lint, ['Project.xcodeproj', 'SomeTarget', '.sources.yml', '--no-delete-config'])

            with open('.swiftlint.yml', 'r') as sources:
                sources = yaml.load(sources)

                self.assertIn('included', sources)
                self.assertEqual(sources['included'], ['A', 'B', 'C'])

    @patch('commands.lint.list_swift_files')
    @patch('commands.lint.run')
    def test_that_swiftlint_is_invoked(self, run, list_files):
        """
        Test that linting invokes SwiftLint.
        """
        list_files.return_value = ['A', 'B', 'C']
        run.return_value.returncode = -8
        swiftlint_path = os.path.join(os.getcwd(), 'tests/resources/.swiftlint.yml')

        with self.runner.isolated_filesystem():
            copyfile(swiftlint_path, '.sources.yml')
            result = self.runner.invoke(lint, ['Project.xcodeproj', 'SomeTarget', '.sources.yml'])

            run.assert_called_once_with(["swiftlint"])
            self.assertEqual(-8, result.exit_code)



import pathlib
from setuptools import setup

HERE = pathlib.Path(__file__).parent
README = (HERE / "README.md").read_text()

setup(
    name='swiftlint-runner',
    version='0.0.1',
    long_description=README,
    long_description_content_type='text/markdown',
    packages=['commands', 'commands.utils'],
    url='https://github.com/elpassion/swiftlint-runner',
    license='MIT',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
    ],
    author='Jakub Turek',
    author_email='jakub.turek@elpassion.pl',
    description='Missing SwiftLint runner with ability to define config file per target',
    entry_points={
        "console_scripts": [
            "slrunner=commands.lint:lint",
        ]
    },
    install_requires=['click', 'pyyaml', 'mod-pbxproj']
)
